from ygoutil.constant import CardType, CardRace, CardAttribute

def _ensureCardType(cardType: CardType):
        def deco(func):
            def wrapper(self: "SQLBuilder", *args):
                raw = func(self, *args)
                if self._checkHas("cardType", cardType):
                    return raw  # 原 SQL 条件
                return (
                    f"({raw} AND {self._cardTypeSQL(cardType)})"
                )  # 附加上 CardType 条件

            return wrapper

        return deco

class SQLBuilder:
    _base = "SELECT t.name,t.DESC,d.* FROM texts t INNER JOIN datas d ON t.id=d.id"
    _names = (
        "keyword",
        "race",
        "attribute",
        "cardType",
        "level",
        "rank",
        "link",
        "Pmark",
        "attack",
        "defence",
        "id",
        "atkDefSum",
        "atkEqDef",
    )

    def __init__(self):
        self.materials = {}
        self.params = []
        self.conditions = []
        self.sql: str = self._base

    def __str__(self):
        return self.resolve()

    def __repr__(self) -> str:
        return f"{self.materials.__repr__()}\n{super().__repr__()}"

    @staticmethod
    def _upperFirst(text: str):
        # 用 str.capitalize 会导致其余字母都变小写 cardType => Cardtype
        return text[:1].upper() + text[1:]

    @staticmethod
    def dealKeyword(text: str) -> str:  # 占位 暂不处理
        return text

    @staticmethod
    def dealRace(text: str) -> CardRace | None:
        if text.endswith("族"):
            text = text[:-1]
        return CardRace.fromStr(text)

    @staticmethod
    def dealAttribute(text: str) -> CardAttribute | None:
        if text.endswith("属性"):
            text = text[:-2]
        return CardAttribute.fromStr(text)

    @staticmethod
    def dealCardType(text: str) -> CardType | None:
        if text.endswith("卡"):
            text = text[:-1]
        base = text[-2:]
        baseType = None
        # if (text.endswith("怪兽") and text!="怪兽") or (text.endswith("魔法") and text!="魔法") or (text.endswith("陷阱") and text!="陷阱"):
        if base in ("怪兽", "魔法", "陷阱") and text != base:
            text = text[:-2]
            baseType = CardType.fromStr(base)
        textType = CardType.fromStr(text)
        if baseType is not None and textType is not None:
            textType |= baseType
        return textType

    @staticmethod
    def _dealAD(text: str) -> int | None:
        if text.isdigit() and len(text) < 5:  # 4 位及以下的数
            return int(text)
        if text == "？" or text == "?":
            return -2

    @classmethod
    def dealAttack(cls, text: str) -> int | None:
        text = text.lower()
        for x in ("atk", "攻击力", "打点", "攻击", "攻"):
            if text.startswith(x):
                text = text[len(x) :]
            if text.endswith(x):
                text = text[: -len(x)]
        return cls._dealAD(text)

    @classmethod
    def dealDefence(cls, text: str) -> int | None:
        text = text.lower()
        for x in ("def", "守备力", "防御力", "防御", "守备", "守", "防"):
            if text.startswith(x):
                text = text[len(x) :]
            if text.endswith(x):
                text = text[: -len(x)]
        return cls._dealAD(text)

    @staticmethod
    def _dealADSum(text: str) -> int | None:
        if text.isdigit():
            val = int(text)
            if val <= 10000:  # 4 位及以下的数，以及特例 10000
                return val
        if text == "？" or text == "?":
            return -4

    @classmethod
    def dealAtkDefSum(cls, text: str) -> int | None:
        text = text.lower()
        for x in (
            "攻守和",
            "攻防和",
            "攻加守",
            "攻加防",
            "攻击加防御",
            "攻击加守备",
            "atk+def",
            "攻守",
            "攻防",
        ):
            if text.startswith(x):
                text = text[len(x) :]
            if text.endswith(x):
                text = text[: -len(x)]
        return cls._dealADSum(text)

    @staticmethod
    def _dealMark(text: str) -> int | None:
        if text.isdigit():
            val = int(text)
            if 0 < val < 15:  # 1-15 的数
                return val

    @classmethod
    def dealLevel(cls, text: str) -> int | None:
        text = text.lower()
        if text.startswith(("等级", "lv")):
            text = text[2:]
        if text.startswith("l"):
            text = text[1:]
        if text.endswith("星"):
            text = text[:-1]
        return cls._dealMark(text)

    @classmethod
    def dealRank(cls, text: str) -> int | None:
        text = text.lower()
        if text.startswith("阶级"):
            text = text[2:]
        if text.startswith("r"):
            text = text[1:]
        if text.endswith("阶"):
            text = text[:-1]
        return cls._dealMark(text)

    @classmethod
    def dealLink(cls, text: str) -> int | None:
        text = text.lower()
        if text.startswith("link"):
            text = text[4:]
        if text.startswith(("连接", "链接")):
            text = text[2:]
        return cls._dealMark(text)

    @classmethod
    def dealPmark(cls, text: str) -> int | None:
        text = text.lower()
        for x in ("灵摆", "刻度", "p"):
            if text.startswith(x):
                text = text[len(x) :]
            if text.endswith(x):
                text = text[: -len(x)]
        return cls._dealMark(text)

    @staticmethod
    def dealId(text: str) -> int | None:
        # print(text)
        if text.isdigit() and len(text) > 4:  # 5 位及以上的数
            return int(text)

    def _addFlag(self, name: str, *args):
        items = self.materials.get(name)
        for i in args:
            if isinstance(i, str):  # 不是 str 的话就不会检查数据
                i = getattr(self, f"deal{self._upperFirst(name)}")(
                    i
                )  # name="race" => self.dealRace(i)
                if i is None:
                    continue
            if not items:
                items = i
            else:
                items |= i
        if items:
            self.materials[name] = items
            return True

    def _addToList(self, name: str, *args):
        items = self.materials.get(name, [])
        for i in args:
            if isinstance(i, str):  # 不是 str 的话就不会检查数据
                i = getattr(self, f"deal{self._upperFirst(name)}")(
                    i
                )  # name="keyword" => self.dealKeyword(i)
                if i is None:
                    continue
            items.append(i)
        if items:
            self.materials[name] = items
            return True

    def _resolveValue(self, name):
        val = self.materials.get(name)
        if not val and val != 0:
            return
        sqlfunc = getattr(self, f"_{name}SQL")
        if isinstance(val, list):  # 多个可能的值
            valsql = " OR ".join(sqlfunc(v) for v in val)
            if len(val) > 1:
                valsql = f"({valsql})"
        else:
            valsql = sqlfunc(val)
        self.conditions.append(valsql)

    def _resolveKeyword(self):
        kws = self.materials.get("keyword")
        if not kws:
            return
        hasOR = False
        for kw in kws:
            kwsql = self._keywordSQL(kw)
            # names=("race","attribute","cardType","attack","defence")
            exception = ("keyword", "atkEqDef")
            for name in (n for n in self._names if n not in exception):
                val = getattr(self, f"deal{self._upperFirst(name)}")(kw)  # self.dealXxx
                if val is not None:
                    sqlfunc = getattr(self, f"_{name}SQL")  # self._xxxSQL
                    kwsql = f"{kwsql} OR {sqlfunc(val)}"
                    hasOR = True
            if hasOR:
                kwsql = f"({kwsql})"
            self.conditions.append(kwsql)

    def _resolveRace(self):
        self._resolveValue("race")

    def _resolveAttribute(self):
        self._resolveValue("attribute")

    def _resolveCardType(self):
        self._resolveValue("cardType")

    def _resolveAttack(self):
        self._resolveValue("attack")

    def _resolveDefence(self):
        self._resolveValue("defence")

    def _resolveLevel(self):
        self._resolveValue("level")

    def _resolveRank(self):
        self._resolveValue("rank")

    def _resolveLink(self):
        self._resolveValue("link")

    def _resolvePmark(self):
        self._resolveValue("Pmark")

    def _resolveId(self):
        self._resolveValue("id")

    def _resolveAtkDefSum(self):
        self._resolveValue("atkDefSum")

    def _resolveAtkEqDef(self):
        if self.materials.get("atkEqDef"):
            self.conditions.append(self._atkEqDefSQL())

    def _checkHas(self, name, *args):
        item = self.materials.get(name)
        if not item:
            return False
        for x in args:
            if x not in item:
                return False
        return True

    def _keywordSQL(self, kw):
        self.params.append(f"%{kw}%")
        # return r"t.name ||'\n' || t.DESC LIKE '%?%'"
        return r"t.name ||'\n' || t.DESC LIKE ?"

    @_ensureCardType(CardType.Monster)
    def _raceSQL(self, race: CardRace):
        return f"d.race & {race.value}!=0"

    @_ensureCardType(CardType.Monster)
    def _attributeSQL(self, attribute: CardAttribute):  # 满足一个属性的就算
        return f"d.attribute & {attribute.value}!=0"

    def _cardTypeSQL(self, cardType: CardType):
        return f"d.type & {cardType.value}={cardType.value}"  # 满足所有种类的才算

    @_ensureCardType(CardType.Monster)
    def _attackSQL(self, attack):
        return f"d.atk={attack}"

    @_ensureCardType(CardType.Monster)
    def _defenceSQL(self, defence):
        return f"d.def={defence}"

    @_ensureCardType(CardType.Monster)
    def _levelSQL(self, level):
        return f"d.level & 15={level}"  # 15->0b1111

    _rankSQL = _levelSQL
    _rankSQL = _ensureCardType(CardType.Xyz)(_rankSQL)

    _linkSQL = _levelSQL
    _linkSQL = _ensureCardType(CardType.Link)(_linkSQL)

    @_ensureCardType(CardType.Monster | CardType.Pendulum)
    def _PmarkSQL(self, Pmark):
        return f"( (d.level & 0xF000000)>>24 ={Pmark} OR (d.level & 0x00F0000)>>16 ={Pmark} )"

    def _idSQL(self, id):
        return f"d.id={id}"

    @_ensureCardType(CardType.Monster)
    def _atkDefSumSQL(self, atkDefSum):
        return f"d.atk+d.def={atkDefSum}"

    @_ensureCardType(CardType.Monster)
    def _atkEqDefSQL(self):
        return "(d.atk=d.def AND d.atk!=-2 AND d.def!=-2)"
        # return f"d.atk=d.def"

    def keyword(self, *args):
        self._addToList("keyword", *args)
        return self

    name = keyword

    def race(self, *args):
        if self._addFlag("race", *args):
            self.cardType(CardType.Monster)
        return self

    def attribute(self, *args):
        if self._addFlag("attribute", *args):
            self.cardType(CardType.Monster)
        return self

    def cardType(self, *args):
        self._addFlag("cardType", *args)
        return self

    def attack(self, *args):
        if self._addToList("attack", *args):
            self.cardType(CardType.Monster)
        return self

    def defence(self, *args):
        if self._addToList("defence", *args):
            self.cardType(CardType.Monster)
        return self

    def level(self, *args):
        if self._addToList("level", *args):
            self.cardType(CardType.Monster)
        return self

    def rank(self, *args):
        if self._addToList("rank", *args):
            self.cardType(CardType.Monster, CardType.Xyz)
        return self

    def link(self, *args):
        if self._addToList("link", *args):
            self.cardType(CardType.Monster, CardType.Link)
        return self

    def Pmark(self, *args):
        if self._addToList("Pmark", *args):
            self.cardType(CardType.Monster, CardType.Pendulum)
        return self

    def id(self, *args):
        self._addToList("id", *args)
        return self

    def atkDefSum(self, *args):
        if self._addToList("atkDefSum", *args):
            self.cardType(CardType.Monster)
        return self

    def atkEqDef(self):
        self.materials["atkEqDef"] = True
        self.cardType(CardType.Monster)
        return self

    def resolve(self):
        self.conditions = []
        self.params = []
        # names=("keyword","race","attribute","cardType","attack","defence","id")
        for name in self._names:
            getattr(self, f"_resolve{self._upperFirst(name)}")()  # self._resolveXxx()
        cons = " AND ".join(self.conditions)
        self.sql = f"{self._base} WHERE {cons}"
        return self.sql


if __name__ == "__main__":
    b = SQLBuilder()
    # b.name("混沌","光","战士族","600","5000").race("机械").cardType(CardType.Monster,CardType.Link,"魔法卡").attribute("炎属性")
    # b.attack(3000,"6900","99999").defence("19.3","?","3000").id("40","10000",5000).level("0星","10星","L6",13)
    # b.name("异色眼","攻守和0")
    b.name("魂", "炎").race("水").race()
    b.cardType("永续陷阱")
    print(b)
    print(b.params)



# coding:utf-8
# import sqlite3
# import configparser
import json
from typing import TYPE_CHECKING
from urllib import parse

import httpx
from bs4 import BeautifulSoup, NavigableString, CData, Tag
# from bs4.element import NavigableString, CData, Tag

try:
    from bs4.element import TemplateString  # 新版bs4需要
except ImportError:
    TemplateString = NavigableString

from ygoutil.card import Card, CardAttribute, CardRace, LinkMark


class OurOCG:
    def __init__(self):
        self.edition = 2

    def SetTranslateEdition(self, ed: str):
        if ed.lower() == "cn":
            self.edition = 0
        elif ed.lower() == "nw":
            self.edition = 1

    ourocgLink = r"https://www.ourocg.cn/"

    def GetHTML(self, url):
        res = httpx.get(url)
        return res.text

    def GetCardHTMLWithJ(self, searchtext, url, searchHTML):
        targeturl = ""
        cardJson = None
        # Pmark=[None,None]
        html = BeautifulSoup(searchHTML, "lxml")
        if html.find_all("title")[0].string.startswith("搜索"):
            scripts = html.find_all("script")
            for s in scripts:
                if str(s.string).strip().startswith("window.__STORE__"):
                    targets = str(s.string[s.string.find("{") :]).strip()[:-1]
                    carddata = json.loads(targets)
                    sts = searchtext.strip()
                    for tar in carddata["cards"]:
                        if (
                            tar["name"] == sts
                            or tar["name_nw"] == sts
                            or tar["name_ja"] == sts
                            or tar["name_en"] == sts
                        ):
                            targeturl = tar["href"].replace("\\", "")
                            cardJson = tar
                            # Pmark=[tar.get("pend_l",None),tar.get("pend_r",None)]
                            # print(Pmark)
                    if targeturl == "" and len(carddata["cards"]) != 0:
                        tar = carddata["cards"][0]
                        targeturl = tar["href"].replace("\\", "")
                        cardJson = tar
                        # Pmark=[tar.get("pend_l",None),tar.get("pend_r",None)]
                    # print(targeturl)
                    break
        else:
            targeturl = url
        return targeturl, cardJson

    def GetCardFromHTML(self, cardHTML, cardJson):
        # print(cardJson)
        html = BeautifulSoup(cardHTML, "lxml")
        isRD = False
        if html.find("div", {"class": "rd-mark"}):
            isRD = True
        div = html.find_all("div", {"class": "val"})
        # divr=[[y for y in x.stripped_strings] for x in div]
        divr = [
            [
                y
                for y in x._all_strings(
                    True, types=(NavigableString, CData, TemplateString)
                )
            ]
            for x in div
        ]
        # 类似 stripped_strings，但新版本需要纳入 TemplateString 才能效果一致
        # print(divr)
        cardtypes = divr[3]
        c = Card(cardtypes)
        if c.isLink:
            linkmark = html.find("div", {"class": "linkMark"})
            if TYPE_CHECKING:
                assert isinstance(linkmark, Tag)
            for mark in linkmark.find_all("i"):
                temp = mark["class"][1].split("_")
                if temp[2] == "on":
                    c.linkmark.add(LinkMark.fromNumber(int(temp[1]), True))
        c.name = divr[0][self.edition]
        jpname = divr[1][0]
        if jpname != "-":
            c.jpname = jpname.replace("・", "·")
        enname = divr[2][0]
        if enname != "-":
            c.enname = enname
        c.isRD = isRD
        c.id = divr[4][0]
        c.url = cardJson["href"]
        c.img = cardJson["img_url"]
        limitnum = 5
        if isRD:
            limitnum = 4
        c.limit = divr[limitnum][0]
        otnum = limitnum + 1
        if divr[otnum]:  # 如果是OCG/TCG专有
            c.ot = divr[otnum][0]
        effectnum = -1
        if c.isMonster:
            c.race = CardRace.fromStr(divr[otnum + 1][0])
            c.attribute = CardAttribute.fromStr(divr[otnum + 2][0])
            if c.isXyz:
                c.rank = OurOCG.dealInt(divr[otnum + 3][0])
                c.level = c.rank
            if c.isP:
                # c.Pmark=Pmark
                c.Pmark = [cardJson.get("pend_l", None), cardJson.get("pend_r", None)]
            if c.isLink:
                c.linknum = OurOCG.dealInt(divr[otnum + 5][0])
                c.level = c.linknum
                c.attack = OurOCG.dealInt(divr[otnum + 4][0])
            else:
                c.level = OurOCG.dealInt(divr[otnum + 3][0])
                c.attack = OurOCG.dealInt(divr[otnum + 4][0])
                c.defence = OurOCG.dealInt(divr[otnum + 5][0])
        L = len(divr[effectnum])
        tempString = divr[effectnum][-1]
        effectlist = [0]
        for x in range(-2, -1 * (L + 1), -1):
            if divr[effectnum][x] == tempString:
                tempnum = -1 - x
                effectlist.append(L - 2 * tempnum)
                effectlist.append(effectlist[1] + tempnum)
                effectlist.append(effectlist[2] + tempnum)
        effects = divr[effectnum][
            effectlist[self.edition] : effectlist[self.edition + 1]
        ]
        effectText = "\n".join(effects)
        c.effect = OurOCG.beautifyText(effectText)
        return c

    def FindCardByName(self, searchtext):
        url = f"{self.ourocgLink}search/{searchtext}"
        searchHTML = self.GetHTML(url)
        targeturl, cardJson = self.GetCardHTMLWithJ(searchtext, url, searchHTML)
        if targeturl:
            cardHTML = self.GetHTML(targeturl)
            return self.GetCardFromHTML(cardHTML, cardJson)
        return None

    async def AsyncGetHTML(self, url):
        async with httpx.AsyncClient() as client:
            res = await client.get(url)
            return res.text

    async def AsyncSearchByName(self, searchtext):
        url = f"{self.ourocgLink}search/{searchtext}"
        searchHTML = await self.AsyncGetHTML(url)
        targeturl, cardJson = self.GetCardHTMLWithJ(searchtext, url, searchHTML)
        if targeturl:
            cardHTML = await self.AsyncGetHTML(targeturl)
            return self.GetCardFromHTML(cardHTML, cardJson)
        return None

    wikiLink = r"https://yugioh-wiki.net/"

    def getWikiLink(self, card):
        if card.jpname:
            pageword = f"《{OurOCG.towikistr(card.jpname)}》"
        elif card.enname:
            pageword = f"《{card.enname}》"
        else:
            return None
        pageword = parse.quote(pageword, encoding="euc-jp")
        return f"{self.wikiLink}index.php?cmd=read&page={pageword}"

    @staticmethod
    def towikistr(text):
        """半角转全角，以及一些特殊符号的转换"""
        transDict = {
            #' ':chr(12288), #半角空格直接转化
            "·": "・",
            "－": "−",
            "Ⅰ": "Ｉ",
            "Ⅱ": "ＩＩ",
            "Ⅲ": "ＩＩＩ",
            "Ⅳ": "ＩＶ",
            "Ⅴ": "Ｖ",
            "Ⅵ": "ＶＩ",
            "Ⅶ": "ＶＩＩ",
            "Ⅷ": "ＶＩＩＩ",
            "Ⅸ": "ＩＸ",
            "Ⅹ": "Ｘ",
            "Ⅺ": "ＸＩ",
            "Ⅻ": "ＸＩＩ",
        }
        r = ""
        for c in text:
            trans = transDict.get(c, None)
            if trans:
                c = trans
            else:
                oc = ord(c)
                if oc > 32 and oc <= 126:  # 半角字符（除空格）根据关系转化
                    c = chr(oc + 65248)
            r += c
        return r

    @staticmethod
    def dealInt(text):
        if text.isdigit():
            return int(text)
        else:
            return text

    @staticmethod
    def beautifyText(text):
        """试着给效果文本加换行，好看一点"""
        nums = set(list("①②③④⑤⑥⑦⑧⑨⑩●"))
        transDict = {"・": "·"}
        r = ""
        length = len(text)
        for i, c in enumerate(text):
            trans = transDict.get(c, None)
            if trans:
                c = trans
            elif c in nums:
                if 1 < i < length - 1 and text[i - 1] != "\n" and text[i + 1] == "：":
                    c = "\n" + c
            r += c
        return r


if __name__ == "__main__":
    # text=input()
    # a=ourocg()
    # print(a.FindCardByName(text))

    async def main():
        text = input()
        a = OurOCG()
        print(await a.AsyncSearchByName(text))

    import asyncio

    asyncio.run(main())




import pytest

from ygoutil.sqlbuilder import SQLBuilder
from ygoutil.constant import CardType

@pytest.fixture
def builder():
    return SQLBuilder()

def test_sql_builder(builder: SQLBuilder):
    # assert not builder.params
    assert builder.name("混沌", "光", "战士族", "600", "5000").race("机械").cardType(CardType.Monster, CardType.Link,"魔法卡").attribute("炎属性").resolve()
    assert builder.sql

def test_sql_builder2(builder: SQLBuilder):
    # assert not builder.params
    assert builder.attack(3000,"6900","99999").defence("19.3","?","3000").id("40","10000",5000).level("0星","10星","L6",13).resolve()

def test_sql_builder3(builder: SQLBuilder):
    # assert not builder.params
    assert builder.name("异色眼","攻守和0").resolve()

def test_sql_builder4(builder: SQLBuilder):
    # assert not builder.params
    assert builder.name("魂", "炎").race("水").race().resolve()

def test_sql_builder5(builder: SQLBuilder):
    # assert not builder.params
    assert builder.cardType("永续陷阱").resolve()






from typing import TYPE_CHECKING, Any
from enum import Enum, IntFlag

def _repr(self: IntFlag):
    return f"{self.name or self.__class__.__name__}({hex(self.value)})"

if TYPE_CHECKING:
    def _repr(self: Any) -> str:
        raise NotImplementedError

class CardType(IntFlag):
    """ 卡片种类 """
    Monster = 0x1
    Spell = 0x2
    Trap = 0x4
    # N/A=0x8
    Normal = 0x10  # 通常（怪兽）
    Effect = 0x20
    Fusion = 0x40
    Ritual = 0x80
    TrapMonster = 0x100
    Spirit = 0x200
    Union = 0x400
    Gemini = 0x800
    Tuner = 0x1000
    Synchro = 0x2000
    Token = 0x4000
    # N/A=0x8000
    Quick_Play = 0x10000
    Continuous = 0x20000
    Equip = 0x40000
    Field = 0x80000
    Counter = 0x100000
    Flip = 0x200000
    Toon = 0x400000
    Xyz = 0x800000
    Pendulum = 0x1000000
    SPSummon = 0x2000000
    Link = 0x4000000

    def __str__(self):
        return cardType2str.get(self) or super().__str__()

    __repr__ = _repr

    @staticmethod
    def fromStr(text: str):
        return str2cardType.get(text)


cardType2str = {
    CardType.Monster: "怪兽",
    CardType.Normal: "通常",
    CardType.Effect: "效果",
    CardType.Tuner: "调整",
    CardType.Gemini: "二重",
    CardType.Flip: "反转",
    CardType.Union: "同盟",
    CardType.Spirit: "灵魂",
    CardType.Toon: "卡通",
    CardType.Fusion: "融合",
    CardType.Synchro: "同调",
    CardType.Xyz: "超量",
    CardType.Pendulum: "灵摆",
    CardType.Ritual: "仪式",
    CardType.Link: "连接",
    CardType.TrapMonster: "陷阱怪兽",
    CardType.SPSummon: "特殊召唤",
    CardType.Token: "衍生物",
    CardType.Spell: "魔法",
    CardType.Trap: "陷阱",
    CardType.Quick_Play: "速攻",
    CardType.Continuous: "永续",
    CardType.Equip: "装备",
    CardType.Field: "场地",
    CardType.Counter: "反击",
}

str2cardType = {v: k for k, v in cardType2str.items()}
str2cardType["XYZ"] = CardType.Xyz


class CardAttribute(IntFlag):
    """ 卡片属性 """
    Earth = 0x01
    Water = 0x02
    Fire = 0x04
    Wind = 0x08
    Light = 0x10
    Dark = 0x20
    Divine = 0x40  # 神属性

    def __str__(self):
        return cardAttribute2str.get(self) or super().__str__()

    __repr__ = _repr

    @staticmethod
    def fromStr(text: str):
        return str2cardAttribute.get(text)


cardAttribute2str = {
    CardAttribute.Earth: "地",
    CardAttribute.Water: "水",
    CardAttribute.Fire: "炎",
    CardAttribute.Wind: "风",
    CardAttribute.Light: "光",
    CardAttribute.Dark: "暗",
    CardAttribute.Divine: "神",
}

str2cardAttribute = {v: k for k, v in cardAttribute2str.items()}


class CardRace(IntFlag):
    """ 卡片种族 """
    Warrior = 0x1
    Spellcaster = 0x2
    Fairy = 0x4
    Fiend = 0x8
    Zombie = 0x10
    Machine = 0x20
    Aqua = 0x40
    Pyro = 0x80
    Rock = 0x100
    WingedBeast = 0x200
    Plant = 0x400
    Insect = 0x800
    Thunder = 0x1000
    Dragon = 0x2000
    Beast = 0x4000
    Beast_Warrior = 0x8000
    Dinosaur = 0x10000
    Fish = 0x20000
    SeaSerpent = 0x40000
    Reptile = 0x80000
    Psychic = 0x100000
    Divine_Beast = 0x200000
    CreatorGod = 0x400000
    Wyrm = 0x800000
    Cyberse = 0x1000000
    Illusion=0x2000000

    def __str__(self):
        return cardRace2str.get(self) or super().__str__()

    __repr__ = _repr

    @staticmethod
    def fromStr(text: str):
        return str2cardRace.get(text)


cardRace2str = {
    CardRace.Warrior: "战士",
    CardRace.Spellcaster: "魔法师",
    CardRace.Fairy: "天使",
    CardRace.Fiend: "恶魔",
    CardRace.Zombie: "不死",
    CardRace.Machine: "机械",
    CardRace.Aqua: "水",
    CardRace.Pyro: "炎",
    CardRace.Rock: "岩石",
    CardRace.WingedBeast: "鸟兽",
    CardRace.Plant: "植物",
    CardRace.Insect: "昆虫",
    CardRace.Thunder: "雷",
    CardRace.Dragon: "龙",
    CardRace.Beast: "兽",
    CardRace.Beast_Warrior: "兽战士",
    CardRace.Dinosaur: "恐龙",
    CardRace.Fish: "鱼",
    CardRace.SeaSerpent: "海龙",
    CardRace.Reptile: "爬虫类",
    CardRace.Psychic: "念动力",
    CardRace.Divine_Beast: "幻神兽",
    CardRace.CreatorGod: "创造神",
    CardRace.Wyrm: "幻龙",
    CardRace.Cyberse: "电子界",
    CardRace.Illusion: "幻想魔"
}

str2cardRace = {v: k for k, v in cardRace2str.items()}
str2cardRace["爬虫"] = CardRace.Reptile


class LinkMark(IntFlag):
    """ 连接标记 """
    BottomLeft = 0x001
    Bottom = 0x002
    BottomRight = 0x004
    Left = 0x008
    # Middle=0x010
    Right = 0x020
    TopLeft = 0x040
    Top = 0x080
    TopRight = 0x100

    def toNumber(self):  # 1-9
        binstr = bin(self.value)[2:]  # "0b01" => "01"  len("01")=2
        return len(binstr)

    def __str__(self):
        return linkMark2str[self.toNumber() - 1]

    __repr__ = _repr

    @staticmethod
    def fromNumber(num, withMid=False):  # num: 1-9
        if withMid and num > 4:
            num -= 1
        # return number2linkMark[num-1]
        return list(LinkMark)[num - 1]


# number2linkMark=[lm for lm in LinkMark]
# linkMark2str=[" ◣"," ↓ ","◢ ","←","   ","→"," ◤"," ↑ ","◥ "]
linkMark2str = ["↙", "⬇", "↘", "⬅", "    ", "➡", "↖", "⬆", "↗"]  # "⏺"


class CardCategory(Enum):
    """ 效果分类 """
    魔陷破坏 = 0x01  # SpellTrapDestroy=0x01
    怪兽破坏 = 0x02  # MonsterDestroy=0x02
    除外 = 0x04  # Remove=0x04
    送去墓地 = 0x08  # ToGrave=0x08
    返回手卡 = 0x10  # ToHand=0x10
    返回卡组 = 0x20  # ToDeck=0x20
    手卡破坏 = 0x40  # DiscardHand=0x40
    卡组破坏 = 0x80  # DiscardDeck=0x80
    抽卡辅助 = 0x100  # Draw=0x100
    卡组检索 = 0x200  # Search=0x200
    卡片回收 = 0x400  # Recycle=0x400
    表示形式 = 0x800  # Position=0x800
    控制权 = 0x1000  # Control=0x1000
    攻守变化 = 0x2000  # AtkDefChange=0x2000
    贯通伤害 = 0x4000  # Brionac=0x4000 #穿刺/贯通
    多次攻击 = 0x8000  # MultiAttack=0x8000
    攻击限制 = 0x10000  # AttackRestrain=0x10000 #攻击限制
    直接攻击 = 0x20000
    特殊召唤 = 0x40000
    衍生物 = 0x80000
    种族相关 = 0x100000
    属性相关 = 0x200000
    效果伤害 = 0x400000
    回复 = 0x800000
    破坏耐性 = 0x1000000
    效果耐性 = 0x2000000
    指示物 = 0x4000000
    赌博 = 0x8000000
    融合相关 = 0x10000000
    同调相关 = 0x20000000
    超量相关 = 0x40000000
    效果无效 = 0x80000000

    def __str__(self):
        return self.name

    __repr__ = _repr


if __name__ == "__main__":
    print(CardType.Token)
    print(list(CardType))
    print(CardType.Xyz | CardType.Pendulum)



from enum import Enum
from typing import Iterable
from functools import partialmethod

from ygoutil.constant import CardType, CardRace, CardAttribute, LinkMark, linkMark2str, CardCategory

class Card:
    """YGO 卡片"""

    def __init__(self, types: Iterable[CardType | str] = None):
        self.id = None  # 卡号
        self.name: str | None = None  # 卡名（中文）
        self.jpname: str | None = None  # 卡名（日文）
        self.enname: str | None = None  # 卡名（英文）
        self.effect: str | None = None  # 效果文本

        self.ot: str | None = None  # ot状态
        self.set = set()  # 系列
        self.cardType: set[CardType | str] = set()  # 卡片类型
        self.limit: str | None = None  # 禁限

        self.category: set[CardCategory] = set()  # 效果种类

        self.alias = None  # 作为同名卡在数据库中的实际卡号

        self.isRD = False  # RD卡

        if types:
            self.fillCardType(*types)
            self.initMonster()  # 根据卡片类型初始化怪兽信息
        # 一系列链接
        self.img: str | None = None  # 卡图链接
        self.url: str | None = None  # 卡片信息来源链接
        self.database: str | None = None
        self.QA: str | None = None
        self.wiki: str | None = None
        self.yugipedia: str | None = None
        # self.ygorg=None
        self.ourocg: str | None = None
        self.script: str | None = None  # 脚本链接
        self.ocgRule: str | None = None

    def __str__(self):
        return self.info()

    def __repr__(self):
        id = self._checkAndFill(self.id, "{}")
        name = self._checkAndFill(self.name, "{}," if id else "{}")
        content = self._checkAndFill(f"{name}{id}", "({})")
        return f"{self.__class__.__name__}{content}"

    @property
    def isMonster(self):
        """ 是怪兽卡 """
        return CardType.Monster in self.cardType

    @property
    def isXyz(self):
        """ 是超量 """ 
        return CardType.Xyz in self.cardType

    @property
    def isP(self):
        """ 是灵摆 """ 
        return CardType.Pendulum in self.cardType

    @property
    def isLink(self):
        """ 是连接 """
        return CardType.Link in self.cardType

    def fillCardType(self, *types: CardType | str):
        for t in types:
            if isinstance(t, str) and (ct := CardType.fromStr(t)):
                self.cardType.add(ct)
            else:
                self.cardType.add(t)

    def initMonster(self):
        if self.isMonster:
            self.attack = None  # 攻击力
            self.defence = None  # 守备力
            self.level = None  # 等级
            self.race = None  # 种族
            self.attribute = None  # 属性
            if self.isXyz:
                self.rank = None  # 阶级
            if self.isP:
                self.Pmark = [None, None]  # 刻度
            if self.isLink:
                self.linknum = None  # link 数
                self.linkmark: set[LinkMark] = set()

    @staticmethod
    def _checkAndFill(text, filltext: str, default=""):
        if text is not None:
            return filltext.format(text)
        return default

    def _infoGen(self):
        yield self._checkAndFill(self.name, "卡名 {}\n")
        yield self._checkAndFill(self.jpname, "日文名 {}\n")
        yield self._checkAndFill(self.enname, "英文名 {}\n")
        if self.cardType:  # 卡片种类
            yield f"{' '.join(str(ct) for ct in self.cardType)}\n"
        if self.isRD:
            yield "RUSH DUEL  "
        else:
            yield self._checkAndFill(self.id, "{}  ")
        yield self._checkAndFill(self.limit, "{}")  # 禁限
        yield self._checkAndFill(self.ot, "  {}\n", "\n")  # O/T
        if self.set:  # 卡片字段
            yield f"系列 {' '.join(self.set)}\n"
        if self.isMonster:
            yield self._checkAndFill(str(self.race), "{}族")
            yield self._checkAndFill(str(self.attribute), "  {}属性")
            if self.isXyz:
                yield self._checkAndFill(self.rank, "  {}阶\n")
            if self.isLink:
                yield self._checkAndFill(self.linknum, "  LINK-{}\n")
                # result+=self._checkAndFill(self.attack,"攻击力 {}\n")
                yield self._checkAndFill(self.attack, "{}/-\n")
                middle = linkMark2str[len(linkMark2str) // 2]
                # marklist=["   "]*8
                # marklist=[middle]*8
                marklist = [
                    str(lm) if lm in self.linkmark else middle for lm in LinkMark
                ]
                marklines = [
                    f"{marklist[5]}{marklist[6]}{marklist[7]}\n",
                    f"{marklist[3]}{middle}{marklist[4]}\n",
                    f"{marklist[0]}{marklist[1]}{marklist[2]}\n",
                ]
                yield "".join(line for line in marklines if line.strip())
            else:
                if not self.isXyz:
                    yield self._checkAndFill(self.level, "  {}星\n")
                # result+=self._checkAndFill(self.attack,"攻击力 {}")
                # result+=self._checkAndFill(self.defence,"  守备力 {}\n")
                yield self._checkAndFill(self.attack, "{}/")
                yield self._checkAndFill(self.defence, "{}\n")
            if self.isP:
                if self.effect and not self.effect.startswith("←"):
                    yield f"←{self.Pmark[0]} 【灵摆】 {self.Pmark[1]}→\n"
        effecttext = self._checkAndFill(self.effect, "{}")
        yield effecttext.replace("・", "·")

    def info(self):
        """ 卡片信息 """
        return "".join(self._infoGen())

    def fromCDBTuple(self, t, setdict: dict = None, lfdict: dict = None):
        self.name = t[0]
        self.effect = t[1]
        self.id = t[2]
        if t[3] == 1:
            self.ot = "OCG专有卡"
        elif t[3] == 2:
            self.ot = "TCG专有卡"
        if t[4] != 0:
            self.alias = self.id
            self.id = t[4]
        if setdict:
            setval = t[5]
            while setval != 0:
                setname = setdict.get(setval & 0xFFFF, None)
                if setname:
                    self.set.add(setname)
                setval = setval >> 16
        if lfdict:
            lfname = ["禁止", "限制", "准限制", "无限制"]
            lfnum = lfdict.get(self.id, 3)
            self.limit = lfname[lfnum]
        self.cardType = Card.bit2CardTypes(t[6])
        if self.isMonster:
            self.attack = Card.dealAtkDef(t[7])
            self.level = Card.dealLevel(t[9])
            if self.isLink:
                self.linkmark = Card.bit2LinkMark(t[8])
                self.linknum = self.level
            else:
                self.defence = Card.dealAtkDef(t[8])
            if self.isXyz:
                self.rank = self.level
            if self.isP:
                self.Pmark = Card.getPmark(t[9])
            self.race = Card.bit2Race(t[10])
            self.attribute = Card.bit2Attribute(t[11])
            self.category = Card.bit2Category(t[12])

    @staticmethod
    def dealAtkDef(val):
        return val if val >= 0 else "?"

    @staticmethod
    def dealLevel(val):
        return val & 0b1111

    @staticmethod
    def getPmark(val):
        pl = (val & 0xF000000) >> 24
        pr = (val & 0x00F0000) >> 16
        return [pl, pr]

    @staticmethod
    def bit2Set(bit, enum: type[Enum]):
        return {x for x in enum if x.value & bit != 0}

    @staticmethod
    def bit2Item(bit, enum: type[Enum]):
        r = None
        if any((r := x) for x in enum if x.value & bit != 0):
            return r
        return None

    # @staticmethod
    # def funcWithEnum(func,enum):
    #     def wrapper(bit):
    #         return func(bit,enum)
    #     return wrapper

    bit2CardTypes: partialmethod[set[CardType | str]] = partialmethod(bit2Set, enum=CardType)
    bit2Race: partialmethod[CardRace] = partialmethod(bit2Item, enum=CardRace)
    bit2Attribute: partialmethod[CardAttribute] = partialmethod(bit2Item, enum=CardAttribute)
    bit2LinkMark: partialmethod[set[LinkMark]] = partialmethod(bit2Set, enum=LinkMark)
    bit2Category: partialmethod[set[CardCategory]] = partialmethod(bit2Set, enum=CardCategory)
    # bit2CardTypes=funcWithEnum.__func__(bit2Set.__func__,CardType)
    # bit2Race=funcWithEnum.__func__(bit2Item.__func__,CardRace)
    # bit2Attribute=funcWithEnum.__func__(bit2Item.__func__,CardAttribute)
    # bit2Linkmark=funcWithEnum.__func__(bit2Set.__func__,LinkMark)
    # bit2Category=funcWithEnum.__func__(bit2Set.__func__,CardCategory)


# Card.bit2CardTypes=Card.funcWithEnum(Card.bit2Set,CardType)



# import json

from typing import TYPE_CHECKING
# from urllib import parse
import httpx
from bs4 import BeautifulSoup, Tag
# from bs4.element import PageElement, Tag
# try:
#     from bs4.element import TemplateString #新版bs4需要
# except:
#     TemplateString=NavigableString

from ygoutil.card import Card, CardAttribute, CardRace, CardType, LinkMark


class BaiGe:
    link = "https://ygocdb.com/"

    linkmark = {
        "↙": LinkMark.BottomLeft,
        "↓": LinkMark.Bottom,
        "↘": LinkMark.BottomRight,
        "←": LinkMark.Left,
        "→": LinkMark.Right,
        "↖": LinkMark.TopLeft,
        "↑": LinkMark.Top,
        "↗": LinkMark.TopRight,
    }

    linkName = {"数据库": "database", "Q&A": "QA", "脚本": "script", "裁定": "ocgRule"}

    @staticmethod
    def imgLink(cardId):
        return f"https://cdn.233.momobako.com/ygopro/pics/{cardId}.jpg"

    @staticmethod
    def dealInt(text: str):
        if text.isdigit():
            return int(text)
        else:
            return text

    def getHTML(self, url):
        res = httpx.get(url, timeout=20)
        return res.text

    async def asyncGetHTML(self, url):
        async with httpx.AsyncClient(timeout=20) as client:
            res = await client.get(url)
            return res.text

    def html2Card(self, html, cardPage=False) -> Card | None:
        html = BeautifulSoup(html, "lxml")
        if cardPage:
            divs = html.find_all("div", {"class": "row card detail"})
        else:
            divs = html.find_all("div", {"class": "row card result"})
        if not divs:
            return None
        cards = []
        for div in divs:
            div: Tag
            info: list[Tag] = div.find_all("div", limit=3, recursive=False)
            if cardPage:
                if TYPE_CHECKING:
                    assert info[2].parent
                info[2] = info[2].parent
            types = None
            data = None
            desc = ""
            state = "type"
            if TYPE_CHECKING:
                assert info[2].div
            for child in info[2].div.children:  # 效果文本
                if state == "type":
                    if isinstance(child, str):
                        child = child.strip()
                        if child:
                            types, *data = child.split()
                            types = types[1:-1].split("|")  # [魔法|永续]
                            state = "data"
                elif state == "data":
                    if isinstance(child, Tag):
                        if child.name == "hr":
                            state = "desc"
                    elif isinstance(child, str):
                        child = child.strip()
                        if child:
                            data = (data or []) + child.split()
                elif state == "desc":
                    if isinstance(child, Tag):
                        if TYPE_CHECKING:
                            assert types
                        if child.name == "br":
                            desc += "\n"
                        elif child.name == "hr" and "灵摆" in types:
                            desc = desc.rstrip()
                            desc += "\n{p}"
                        else:
                            child = child.text
                    if isinstance(child, str):
                        desc += child
            if not types:
                return None
            c = Card(types)
            if TYPE_CHECKING:
                assert info[1].h2
            c.name = info[1].h2.text
            extras = info[1].find_all("h3")
            # print(extras)
            c.id = extras.pop().findChild("span").text

            if len(extras) < 2:
                if extras[-1].text.isascii():
                    c.enname = extras.pop().text
                else:
                    c.jpname = extras.pop().text
            else:
                c.enname = extras.pop().text
                c.jpname = extras.pop().text
            # c.database,c.QA,c.wiki,c.yugipedia,c.ygorg,c.ourocg,c.script,c.ocgRule=[
            #     a.get("href") for a in info[1].find_all("a")]
            # links={ a.text.strip().lower() : a.get("href") for a in info[1].find_all("a")}
            for atag in info[1].find_all("a"):
                aname: str = atag.text.strip().lower()
                alink: str = atag.get("href")
                if aname in self.linkName:
                    setattr(c, self.linkName[aname], alink)
                elif hasattr(c, aname):
                    setattr(c, aname, alink)
                elif aname.startswith("详情"):
                    if alink.endswith("#faq"):
                        alink = alink[:-4]
                    c.url = f"{self.link[:-1]}{alink}"  # https://ygocdb.com/card/xxxx

            # c.database=links.get("数据库")
            # c.QA=links.get("q&a")
            # c.wiki=links.get("wiki")
            # c.yugipedia=links.get("yugipedia")
            # # c.ygorg=links.get("ygorg") # 好像不再提供这个链接了
            # c.ourocg=links.get("ourocg")
            # c.script=links.get("脚本")
            # c.ocgRule=links.get("裁定")
            # print(links)
            if TYPE_CHECKING:
                assert info[0].img
            img = info[0].img.get("data-original")
            if TYPE_CHECKING:
                assert isinstance(img, str)
            c.img = img
            if isinstance(c.img, str) and c.img.endswith("!half"):
                c.img = c.img[:-5]
            if c.isMonster:
                if TYPE_CHECKING:
                    assert data
                race, attribute = data.pop(0).split("/")  # 恐龙/地
                c.race = CardRace.fromStr(race)
                c.attribute = CardAttribute.fromStr(attribute)
                lv = ""
                for char in data.pop(0):  # [★2] [☆4] [LINK-3]
                    if char.isdigit():
                        lv += char
                lv = BaiGe.dealInt(lv)
                c.level = lv
                attack, defence = data.pop(0).split("/")  # 1400/1200
                c.attack = BaiGe.dealInt(attack)
                c.defence = BaiGe.dealInt(defence)
                if c.isXyz:
                    c.rank = lv
                if c.isLink:
                    c.linknum = lv
                    c.defence = None
                    marks = data.pop(0).split("]")  # [↑][←][→][↙][↓][↘]
                    for m in marks:
                        m = m.strip("[")
                        if m:
                            c.linkmark.add(BaiGe.linkmark[m])
                if c.isP:
                    if data:
                        marks = data.pop(0).split("/")  # 4/4
                    else:
                        # 刻度是 0/0 时好像不会展示在网页上
                        marks = ["0", "0"]
                    c.Pmark = [BaiGe.dealInt(m) for m in marks]
                    desc = f"←{marks[0]} 【灵摆】 {marks[1]}→" + desc
                    if CardType.Normal in c.cardType:
                        desc = desc.format(p="【怪兽描述】")
                    else:
                        desc = desc.format(p="【怪兽效果】")
            c.effect = desc.strip()
            cards.append(c)

            return c  # 先只找一张卡

    def search(self, text, byID=False):
        if byID:
            url = f"{self.link}card/{text}"  # text是卡号
        else:
            url = f"{self.link}?search={text}"
        html = self.getHTML(url)
        return self.html2Card(html, cardPage=byID)

    async def asyncSearch(self, text, byID=False):
        if byID:
            url = f"{self.link}card/{text}"  # text是卡号
        else:
            url = f"{self.link}?search={text}"
        html = await self.asyncGetHTML(url)
        return self.html2Card(html, cardPage=byID)

    def getJSON(self, url):
        res = httpx.get(url)
        return res.json()

    async def asyncGetJSON(self, url):
        async with httpx.AsyncClient() as client:
            res = await client.get(url)
            return res.json()

    def json2Card(self, rjson):
        # TODO
        pass

    def searchAPI(self, text):
        url = f"{self.link}api/v0/?search={text}"
        rjson = self.getJSON(url)
        return self.json2Card(rjson)

    async def asyncSearchAPI(self, text):
        url = f"{self.link}api/v0/?search={text}"
        rjson = await self.asyncGetJSON(url)
        return self.json2Card(rjson)


if __name__ == "__main__":

    async def main():
        text = input()
        a = BaiGe()
        print(await a.asyncSearch(text))

    import asyncio

    asyncio.run(main())



import random
from pathlib import Path

import tomli
import tomli_w

from ygoutil.dataloader import CDBReader

class YGORoom:
    ygodir = Path()

    roomFile = "MemberRoom.toml"
    memberRooms = {}
    servers = {}

    boolCodeMap = {
        "match": "M",
        "tag": "T",
        "tcg": "TO",
        "ot": "OT",
        "nolflist": "NF",
        "nounique": "NU",
        "nocheck": "NC",
        "noshuffle": "NS",
        "ai": "AI",
    }
    intCodeMap = {
        "lp": ("LP", (1, 99999, 8000)),
        "time": ("TM", (0, 999, 3)),
        "start": ("ST", (1, 40, 5)),
        "draw": ("DR", (0, 35, 1)),
        "lflist": ("LF", (1, 99999, 1)),
        "rule": ("MR", (1, 5, 5)),
    }
    # intRangeMap={"lp":(1,99999,8000),"time":(0,999,3),"start":(1,40,5),"draw":(0,35,1),"lflist":(1,99999,1),"rule":(1,5,5)}
    # (下限，上限，默认值) 禁卡表数量一直在变化，故不设上限

    roomSuffix = (
        "坊",
        "村",
        "城",
        "域",
        "道",
        "屋",
        "馆",
        "现实",
        "居室",
        "空间",
        "之间",
        "的房",
    )

    @classmethod
    def initDuel(cls, ygodir, servers):
        ygodir = Path(ygodir)
        roomFilePath = ygodir / cls.roomFile
        if roomFilePath.exists():
            with open(roomFilePath, "rb") as f:
                cls.memberRooms = tomli.load(f)
        cls.servers = servers
        cls.ygodir = ygodir

    @classmethod
    def saveDuel(cls):
        if not cls.memberRooms:
            return
        roomFilePath = cls.ygodir / cls.roomFile
        with open(roomFilePath, "wb") as f:
            tomli_w.dump(cls.memberRooms, f)

    @classmethod
    def parseRoom(cls, roomText: str):
        prefix = set()
        nameList = roomText.split("#")
        *prefixList, name = nameList
        if prefixList:
            prefix.update(prefixList[0].split(","))
            name = "#".join(nameList[1:])
        return YGORoom(name, prefix)

    @classmethod
    def getMemberRoom(cls, key):
        roomInfo: dict | None = cls.memberRooms.get(key)
        if roomInfo:
            room = cls.parseRoom(roomInfo["room"])
            room.serverName = roomInfo.get("server")
            return room
        return None

    @classmethod
    def saveMemberRoom(cls, key, room: "YGORoom", name=None):
        info = room.roomInfo
        if name:
            info["name"] = name
        cls.memberRooms[key] = info

    @classmethod
    def removeMemberRoom(cls, key):
        if key in cls.memberRooms:
            cls.memberRooms.pop(key)
            return True
        return False

    @classmethod
    def hint(cls, action="记录", key=None, name=None):
        if key:
            return f"{action}了房间【{key}】"
        if name:
            return f"{action}了{name}的房间"
        return f"{action}了房间"

    @classmethod
    def randomRoomName(cls, cdb: CDBReader):
        result = []
        with cdb:
            ct = cdb.getRandomNames(count=random.randint(1, 4))
            result = [random.choice(n) for n in ct]
        return "".join(result) + random.choice(cls.roomSuffix)

    def __init__(self, name=None, prefix=None):
        self.prefix = prefix or set()
        self.name = name or ""
        self.serverName = ""
        self._host = None
        self._port = None

    # def randomRoomName(self,cdb:cdbReader):
    #     result=[]
    #     with cdb:
    #         ct=cdb.getRandomNames(count=random.randint(1,4))
    #         result=[random.choice(n) for n in ct]
    #     self.name="".join(result)+random.choice(self.roomSuffix)
    #     return self.name

    def togglePrefix(self, code):
        if code in self.prefix:
            self.prefix.remove(code)
        else:
            self.prefix.add(code)

    def args2prefix(self, args: dict):
        for arg in args:
            code = self.boolCodeMap.get(arg)
            if code:
                self.togglePrefix(code)
            else:
                code = self.intCodeMap.get(arg)
                if code:
                    code, intRange = code
                    minVal, maxVal, defaultVal = intRange
                    val = args.get(arg)
                    if val is not None:
                        if isinstance(val, str) and val.isdigit():
                            val = int(val)
                        val = max(minVal, val)
                        val = min(maxVal, val)
                        prefix = f"{code}{val}"
                        self.togglePrefix(prefix)
                        for p in [
                            p for p in self.prefix if p.startswith(code) and p != prefix
                        ]:
                            self.prefix.remove(p)

    def server2HostPort(self):
        noserver = (None, None)
        if self.serverName.startswith("233"):
            host, port = self.servers.get("233", noserver)
            port = int(f"2{'3'*self.serverName.count('3')}")
        elif self.serverName.endswith("编年史") or self.serverName.lower() == "dc":
            host, port = self.servers.get("编年史", noserver)
        elif self.serverName == "2pick" or self.serverName == "轮抽":
            host, port = self.servers.get("2pick", noserver)
        elif self.serverName.startswith("复读") or self.serverName.lower() == "repiko":
            host, port = self.servers.get("repiko", noserver)
        else:
            if self.serverName:
                host, port = self.servers.get(self.serverName, noserver)
            else:
                host, port = noserver
        self._host, self._port = host, port

    @property
    def hasServer(self):
        return self.host and self.port

    @property
    def full(self):
        if self.prefix:
            return f"{','.join(self.prefix)}#{self.name}"
        return self.name

    @property
    def server(self):
        if self.hasServer:
            return f"{self.host}  {self.port}"
        return ""

    @property
    def host(self):
        if not self._host and self.serverName:
            self.server2HostPort()
        return self._host

    @property
    def port(self):
        if not self._port and self.serverName:
            self.server2HostPort()
        return self._port

    @property
    def roomInfo(self):
        return {"room": self.full, "server": self.serverName}




import pytest

from ygoutil.source import OurOCG

@pytest.mark.asyncio
async def test_baige():
    source = OurOCG()
    card = await source.AsyncSearchByName("解码语者")
    assert card
    assert card.name == "解码语者"
